

// demonstration program for talking to an AVR microcontroller running the avr_eth code
// copyright 2014, Pieter-Tjerk de Boer, pa3fwm@amsat.org
// licensed under GPLv2

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>


#define AVR_UDP_portnumber 6668
#define AVR_IPaddress "192.168.178.70"


int so;    // will refer to our socket


void send_to_avr(unsigned char *p)
{
   unsigned char b[2048];
   unsigned char *w=b;
   int i,j,k;
   for (i=0;i<20;i++) {
      for (k=0;k<8;k++) *w++=0x55;
      for (j=0;j<8;j++) {
         if (p[0]&(1<<j)) for (k=0;k<8;k++) *w++=0xaa;
         else for (k=0;k<8;k++) *w++=0x55;
      }
      p++;
   }
   for (k=0;k<8;k++) *w++=0x55;
   send(so,b,w-b,MSG_EOR);
}


int recv_from_avr(void)
{
   unsigned char r[1024];
   if (recv(so,r,1024,MSG_DONTWAIT)<=0) return -1;
   int v=0;
   int i;
   for (i=0;i<8;i++) if (r[4+5*i]) v|=(1<<i);   // assemble the user data bits into a byte
   return v;
}


void main(int argc)
{
   // prepare socket
   struct sockaddr_in addr;
   so = socket(PF_INET, SOCK_DGRAM, 0);
   memset(&addr, 0, sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_port = htons(AVR_UDP_portnumber);
   bind(so, (struct sockaddr*)&addr, sizeof(addr));
   addr.sin_port = htons(AVR_UDP_portnumber);
   inet_aton(AVR_IPaddress,&addr.sin_addr);
   connect(so, (struct sockaddr*)&addr, sizeof(addr));

   int d=0;
   // endless loop: transmit a packet every half second, and show any received packets
   for (;;) {
      unsigned char b[20];
      int i;
      for (i=0;i<20;i++) b[i]=i+d;
      if (argc<2) {
        send_to_avr(b);
        printf("------ sent data: %02x ... %02x\n",d,d+19);
      }
      d++;

      for (i=0;i<50;i++) {   // wait total of 50*10=500 ms
         usleep(10000);      // wait 10 ms
         int v;
         while ((v=recv_from_avr())>=0) {
            printf("recv'd byte: %02x\n",v);
         }
      }
   }
}

